;**************************************************************************************************************************************************
;   PROJECT:      Generatore di Funzioni DSS
;   FILE:         Calc.asm
;   LAST REV:     1.2
;	AUTHOR:       Ivo Colleoni
;  	DEVICE:       PIC18F2550
;	CREATED:	  23/01/2013
;	UPDATED:	  04/04/2013
;
;	DESCRIP:      Moltiplicazione e conversione decimale/esadecimale
;   FUNCTION:     - MOLTIP
;                 - BCD9_TO_HEX4
;                 - HEX1_TO_BCD3
;                 - RESET_VAR_CALC
;--------------------------------------------------------------------------------------------------------------------------------------------------
;   Change History:
;  	Rev.    Date                Changes
;   1.2     04/04/2013          - aggiunta funzione HEX1_TO_BCD3
;   1.1     29/01/2013          - correzione bug
;   1.0     23/01/2013          - prima build
;
;**************************************************************************************************************************************************
;dichiarazione variabili, memoria allocata da 0x08 a 0x21
;**************************************************************************************************************************************************
                    CBLOCK              0x08
					freq_moltip_temp_0				;LSB            					;Risultati temporanei moltiplicazione
					freq_moltip_temp_1
					freq_moltip_temp_2
					freq_moltip_temp_3
					freq_moltip_temp_4
					freq_moltip_temp_5
					
					freq_moltip_0       			;LSB            					;Risultati moltiplicazione
					freq_moltip_1       
					freq_moltip_2       
					freq_moltip_3       
					freq_moltip_4       
					freq_moltip_5       
					freq_moltip_6       
					freq_moltip_7       
					freq_moltip_8       
					
					freq_moltip_d_0					;LSB            					;Resto moltiplicazione per arrotondamento
					freq_moltip_d_1		
					freq_moltip_d_2

                    freq_hex_0
                    freq_hex_1
                    freq_hex_2
                    freq_hex_3

                    bcd_unit
                    bcd_dec
                    bcd_cent
                    hex_data
                    ENDC
;-------------------MOLTIPLICATORE-----------------------------------------------------------------------------------------------------------------
;MCLK=16,384MHZ K=Fmclk/2^28=0.061035 M=1/K=16,384
;VALORE MASSIMO IN OGNI VARIABILE M = 9------------------------------------------------------------------------------------------------------------
;		
					CONSTANT	M0 = 2         		;LSB          						;Variabili moltiplicatore
					CONSTANT	M1 = 9
					CONSTANT	M2 = 1
					CONSTANT	M3 = 8
					CONSTANT	M4 = 0
;
;**************************************************************************************************************************************************
;   FUNCTION:       MOLTIP
;   PreCondition:   Valori frequenza inseriti nelle variabili di input
;   Input:          freq_ins_0, freq_ins_1, freq_ins_2, freq_ins_3, freq_ins_4, freq_ins_5, freq_ins_6 (freq_ins_0: LSB)
;                   M0, M1, M2, M3, M4 (M0:LSB)
;   Output:         freq_moltip_0 freq_moltip_1 freq_moltip_2 freq_moltip_3 freq_moltip_4 freq_moltip_5 freq_moltip_6 freq_moltip_7
;                   freq_moltip_8 (freq_moltip_0: LSB)
;   Overview:       Moltiplica 7 byte BCD x 5 byte BCD (di cui 2 sono decine e unit e 3 sono numeri decimali dopo la virgola), salva i risultati in
;                   9 byte BCD (freq_moltip_x) + 3 byte decimali (freq_moltip_d_x) che vengono arrotondati all'LSB di Freq_moltip_x
;                   Moltiplica i valori della frequenza inseriti, divisi in 7 byte da 4 bit massimo (0-9) freq_ins_x per i valori del moliplicatore
;                   Mx trattandoli come se fossero dei numeri decimali (BCD)
;                   Salva i risultati in 9 byte da 4 bit massimo (0-9) freq_moltip_x gestendoli come se fossero un unico numero decimale (BCD),
;                   arrotondamento alla prima unit
;**************************************************************************************************************************************************
;
MOLTIP
;-------------------MOLTIPLICAZIONE PRIMA CIFRA----------------------------------------------------------------------------------------------------
					movlw				M0
					mulwf				freq_ins_0
				    movff				PRODL,freq_moltip_temp_0
				    
				    movlw				M1
					mulwf				freq_ins_0
				    movff				PRODL,freq_moltip_temp_1
				    
				    movlw				M2
					mulwf				freq_ins_0
				    movff				PRODL,freq_moltip_temp_2
				    
				    movlw				M3
					mulwf				freq_ins_0
				    movff				PRODL,freq_moltip_temp_3
				    
				    movlw				M4
					mulwf				freq_ins_0
				    movff				PRODL,freq_moltip_temp_4
;-------------------RIPORTI PRIMA CIFRA------------------------------------------------------------------------------------------------------------
				    decf                freq_moltip_temp_1                				;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_0,W                			;sposto in w per equilibrare il ciclo
		    
CICLO_RIP   		movwf               freq_moltip_temp_0                  			;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
           			incf                freq_moltip_temp_1                  			;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
            
            		movlw				h'A'
					subwf				freq_moltip_temp_0,W
			
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
            		goto                CICLO_RIP                                       ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
				
					movff               freq_moltip_temp_0,freq_moltip_d_0              ;e salva il valore restante nel primo byte dei decimali del risultato
			
					decf                freq_moltip_temp_2                              ;decremento per equilibrare il ciclo
		  		 	movf                freq_moltip_temp_1,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIP_2			movwf               freq_moltip_temp_1                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_2                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_1,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIP_2                                     ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movff               freq_moltip_temp_1,freq_moltip_d_1              ;e salva il valore restante nel secondo byte dei decimali risultato
					
					decf                freq_moltip_temp_3                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_2,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIP_3 		movwf               freq_moltip_temp_2                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_3                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_2,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIP_3                                     ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movff               freq_moltip_temp_2,freq_moltip_d_2              ;e salva il valore restante nel terzo byte dei decimali  risultato
					
		            decf                freq_moltip_temp_4                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_3,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIP_4 		movwf               freq_moltip_temp_3                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_4                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_3,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIP_4                                     ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movff               freq_moltip_temp_3,freq_moltip_0                ;e salva il valore restante nel primo byte del risultato
		
		            decf                freq_moltip_temp_5                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_4,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIP_5			movwf               freq_moltip_temp_4                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_5                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_4,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIP_5                                     ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movff               freq_moltip_temp_4,freq_moltip_1                ;e salva il valore restante nel secondo byte del risultato
					movff               freq_moltip_temp_5,freq_moltip_2                ;e salva il valore restante nel terzo byte risultato (solo qualche riporto al massimo)
 ;-------------------RESET VARIABILI----------------------------------------------------------------------------------------------------------------
					clrf                freq_moltip_temp_0
					clrf                freq_moltip_temp_1
					clrf                freq_moltip_temp_2
					clrf                freq_moltip_temp_3
					clrf                freq_moltip_temp_4
					clrf                freq_moltip_temp_5
;-------------------MOLTIPLICAZIONE SECONDA CIFRA--------------------------------------------------------------------------------------------------
					movlw				M0
					mulwf				freq_ins_1
				    movff				PRODL,freq_moltip_temp_0
				    
				    movlw				M1
					mulwf				freq_ins_1
				    movff				PRODL,freq_moltip_temp_1
				    
				    movlw				M2
					mulwf				freq_ins_1
				    movff				PRODL,freq_moltip_temp_2
				    
				    movlw				M3
					mulwf				freq_ins_1
				    movff				PRODL,freq_moltip_temp_3
				    
				    movlw				M4
					mulwf				freq_ins_1
				    movff				PRODL,freq_moltip_temp_4
;-------------------RIPORTI SECONDA CIFRA----------------------------------------------------------------------------------------------------------
				    decf                freq_moltip_temp_1                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_0,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPb  		movwf               freq_moltip_temp_0                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
            		incf                freq_moltip_temp_1                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
            
          			movlw				h'A'
					subwf				freq_moltip_temp_0,W
			
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
    		        goto                CICLO_RIPb                                      ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_0,W
					addwf				freq_moltip_d_1
					
					decf                freq_moltip_temp_2                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_1,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPb_2		movwf               freq_moltip_temp_1                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
            		incf                freq_moltip_temp_2                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
            		
            		movlw				h'A'
					subwf				freq_moltip_temp_1,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
            		goto                CICLO_RIPb_2                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_1,W
					addwf				freq_moltip_d_2
		
					decf                freq_moltip_temp_3                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_2,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPb_3 		movwf               freq_moltip_temp_2                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
            		incf                freq_moltip_temp_3                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
            		
            		movlw				h'A'
					subwf				freq_moltip_temp_2,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
            		goto                CICLO_RIPb_3                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_2,W
					addwf				freq_moltip_0
					
            		decf                freq_moltip_temp_4                              ;decremento per equilibrare il ciclo
		    		movf                freq_moltip_temp_3,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPb_4 		movwf               freq_moltip_temp_3                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
     		        incf                freq_moltip_temp_4                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
     		       
     		        movlw				h'A'
					subwf				freq_moltip_temp_3,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
    		        goto                CICLO_RIPb_4                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_3,W
					addwf				freq_moltip_1									;sommo il valore ottenuto al secondo byte del risultato
    		        
					decf                freq_moltip_temp_5                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_4,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPb_5		movwf               freq_moltip_temp_4                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
       		    	incf                freq_moltip_temp_5                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
       		     
       		     	movlw				h'A'
					subwf				freq_moltip_temp_4,W
			
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPb_5                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_4,W
					addwf				freq_moltip_2									;sommo il valore ottenuto al terzo byte del risultato
		
					movf				freq_moltip_temp_5,W
					addwf				freq_moltip_3
						
					call				CTRL_RIP_RISULT									;controllo i ripirti sul risultato, se lo faccio dopo ogni operazione al massimo avr? un riporto da gestire (9+9 max)
;-------------------RESET VARIABILI----------------------------------------------------------------------------------------------------------------
					clrf                freq_moltip_temp_0
					clrf                freq_moltip_temp_1
					clrf                freq_moltip_temp_2
					clrf                freq_moltip_temp_3
					clrf                freq_moltip_temp_4
					clrf                freq_moltip_temp_5
;-------------------MOLTIPLICAZIONE TERZA CIFRA----------------------------------------------------------------------------------------------------
					movlw				M0
					mulwf				freq_ins_2
				    movff				PRODL,freq_moltip_temp_0
				    
				    movlw				M1
					mulwf				freq_ins_2
				    movff				PRODL,freq_moltip_temp_1
				    
				    movlw				M2
					mulwf				freq_ins_2
				    movff				PRODL,freq_moltip_temp_2
				    
				    movlw				M3
					mulwf				freq_ins_2
				    movff				PRODL,freq_moltip_temp_3
				    
				    movlw				M4
					mulwf				freq_ins_2
				    movff				PRODL,freq_moltip_temp_4
;-------------------RIPORTI TERZA CIFRA------------------------------------------------------------------------------------------------------------
				    decf                freq_moltip_temp_1                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_0,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPc			movwf               freq_moltip_temp_0                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_1                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_0,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPc                                      ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_0,W
					addwf				freq_moltip_d_2
					
					decf                freq_moltip_temp_2                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_1,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPc_2		movwf               freq_moltip_temp_1                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_2                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_1,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPc_2                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_1,W
					addwf				freq_moltip_0
		
					decf                freq_moltip_temp_3                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_2,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPc_3 		movwf               freq_moltip_temp_2                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_3                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_2,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPc_3                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_2,W
					addwf				freq_moltip_1
					
		            decf                freq_moltip_temp_4                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_3,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPc_4 		movwf               freq_moltip_temp_3                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_4                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_3,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPc_4                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_3,W
					addwf				freq_moltip_2
		            
					decf                freq_moltip_temp_5                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_4,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPc_5		movwf               freq_moltip_temp_4                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_5                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_4,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPc_5                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_4,W
					addwf				freq_moltip_3									;sommo il valore ottenuto al terzo byte del risultato
		
					movf				freq_moltip_temp_5,W
					addwf				freq_moltip_4
						
					call				CTRL_RIP_RISULT									;controllo i ripirti sul risultato, se lo faccio dopo ogni operazione al massimo avr? un riporto da gestire (9+9 max)
;-------------------RESET VARIABILI----------------------------------------------------------------------------------------------------------------
					clrf                freq_moltip_temp_0
					clrf                freq_moltip_temp_1
					clrf                freq_moltip_temp_2
					clrf                freq_moltip_temp_3
					clrf                freq_moltip_temp_4
					clrf                freq_moltip_temp_5
;-------------------MOLTIPLICAZIONE QUARTA CIFRA---------------------------------------------------------------------------------------------------
					movlw				M0
					mulwf				freq_ins_3
				    movff				PRODL,freq_moltip_temp_0
				    
				    movlw				M1
					mulwf				freq_ins_3
				    movff				PRODL,freq_moltip_temp_1
				    
				    movlw				M2
					mulwf				freq_ins_3
				    movff				PRODL,freq_moltip_temp_2
				    
				    movlw				M3
					mulwf				freq_ins_3
				    movff				PRODL,freq_moltip_temp_3
				    
				    movlw				M4
					mulwf				freq_ins_3
				    movff				PRODL,freq_moltip_temp_4
;-------------------RIPORTI QUARTA CIFRA-----------------------------------------------------------------------------------------------------------
				    decf                freq_moltip_temp_1                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_0,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPd			movwf               freq_moltip_temp_0                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_1                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_0,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPd                                      ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_0,W
					addwf				freq_moltip_0
					
					decf                freq_moltip_temp_2                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_1,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPd_2		movwf               freq_moltip_temp_1                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_2                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_1,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPd_2                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_1,W
					addwf				freq_moltip_1
		
					decf                freq_moltip_temp_3                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_2,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPd_3 		movwf               freq_moltip_temp_2                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_3                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_2,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPd_3                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_2,W
					addwf				freq_moltip_2
					
		            decf                freq_moltip_temp_4                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_3,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPd_4 		movwf               freq_moltip_temp_3                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_4                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_3,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPd_4                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_3,W
					addwf				freq_moltip_3
		            
					decf                freq_moltip_temp_5                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_4,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPd_5		movwf               freq_moltip_temp_4                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_5                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_4,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPd_5                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_4,W
					addwf				freq_moltip_4									;sommo il valore ottenuto al terzo byte del risultato
		
					movf				freq_moltip_temp_5,W
					addwf				freq_moltip_5
						
					call				CTRL_RIP_RISULT									;controllo i ripirti sul risultato, se lo faccio dopo ogni operazione al massimo avr? un riporto da gestire (9+9 max)
;------------------RESET VARIABILI-----------------------------------------------------------------------------------------------------------------
					clrf                freq_moltip_temp_0
					clrf                freq_moltip_temp_1
					clrf                freq_moltip_temp_2
					clrf                freq_moltip_temp_3
					clrf                freq_moltip_temp_4
					clrf                freq_moltip_temp_5
;-------------------MOLTIPLICAZIONE QUINTA CIFRA---------------------------------------------------------------------------------------------------
					movlw				M0
					mulwf				freq_ins_4
				    movff				PRODL,freq_moltip_temp_0
				    
				    movlw				M1
					mulwf				freq_ins_4
				    movff				PRODL,freq_moltip_temp_1
				    
				    movlw				M2
					mulwf				freq_ins_4
				    movff				PRODL,freq_moltip_temp_2
				    
				    movlw				M3
					mulwf				freq_ins_4
				    movff				PRODL,freq_moltip_temp_3
				    
				    movlw				M4
					mulwf				freq_ins_4
				    movff				PRODL,freq_moltip_temp_4
;-------------------RIPORTI QUINTA CIFRA-----------------------------------------------------------------------------------------------------------
				    decf                freq_moltip_temp_1                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_0,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPe			movwf               freq_moltip_temp_0                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_1                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_0,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPe                                      ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_0,W
					addwf				freq_moltip_1
					
					decf                freq_moltip_temp_2                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_1,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPe_2		movwf               freq_moltip_temp_1                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_2                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_1,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPe_2                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_1,W
					addwf				freq_moltip_2
		
					decf                freq_moltip_temp_3                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_2,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPe_3 		movwf               freq_moltip_temp_2                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_3                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_2,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPe_3                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_2,W
					addwf				freq_moltip_3
					
		            decf                freq_moltip_temp_4                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_3,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPe_4 		movwf               freq_moltip_temp_3                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_4                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_3,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPe_4                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_3,W
					addwf				freq_moltip_4
		            
					decf                freq_moltip_temp_5                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_4,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPe_5		movwf               freq_moltip_temp_4                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_5                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_4,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPe_5                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_4,W
					addwf				freq_moltip_5									;sommo il valore ottenuto al terzo byte del risultato
		
					movf				freq_moltip_temp_5,W
					addwf				freq_moltip_6
						
					call				CTRL_RIP_RISULT									;controllo i ripirti sul risultato, se lo faccio dopo ogni operazione al massimo avr? un riporto da gestire (9+9 max)
;-------------------RESET VARIABILI----------------------------------------------------------------------------------------------------------------
					clrf                freq_moltip_temp_0
					clrf                freq_moltip_temp_1
					clrf                freq_moltip_temp_2
					clrf                freq_moltip_temp_3
					clrf                freq_moltip_temp_4
					clrf                freq_moltip_temp_5
;-------------------MOLTIPLICAZIONE SESTA CIFRA----------------------------------------------------------------------------------------------------
					movlw				M0
					mulwf				freq_ins_5
				    movff				PRODL,freq_moltip_temp_0
				    
				    movlw				M1
					mulwf				freq_ins_5
				    movff				PRODL,freq_moltip_temp_1
				    
				    movlw				M2
					mulwf				freq_ins_5
				    movff				PRODL,freq_moltip_temp_2
				    
				    movlw				M3
					mulwf				freq_ins_5
				    movff				PRODL,freq_moltip_temp_3
				    
				    movlw				M4
					mulwf				freq_ins_5
				    movff				PRODL,freq_moltip_temp_4
;-------------------RIPORTI SESTA CIFRA------------------------------------------------------------------------------------------------------------
				    decf                freq_moltip_temp_1                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_0,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPf			movwf               freq_moltip_temp_0                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_1                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_0,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPf                                      ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_0,W
					addwf				freq_moltip_2
					
					decf                freq_moltip_temp_2                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_1,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPf_2		movwf               freq_moltip_temp_1                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_2                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_1,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPf_2                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_1,W
					addwf				freq_moltip_3
		
					decf                freq_moltip_temp_3                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_2,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPf_3 		movwf               freq_moltip_temp_2                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_3                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_2,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPf_3                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_2,W
					addwf				freq_moltip_4
					
		            decf                freq_moltip_temp_4                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_3,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPf_4 		movwf               freq_moltip_temp_3                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_4                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_3,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPf_4                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_3,W
					addwf				freq_moltip_5
		            
					decf                freq_moltip_temp_5                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_4,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPf_5		movwf               freq_moltip_temp_4                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_5                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_4,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPf_5                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_4,W
					addwf				freq_moltip_6									;sommo il valore ottenuto al terzo byte del risultato
		
					movf				freq_moltip_temp_5,W
					addwf				freq_moltip_7
						
					call				CTRL_RIP_RISULT									;controllo i ripirti sul risultato, se lo faccio dopo ogni operazione al massimo avr? un riporto da gestire (9+9 max)
;-------------------RESET VARIABILI----------------------------------------------------------------------------------------------------------------
					clrf                freq_moltip_temp_0
					clrf                freq_moltip_temp_1
					clrf                freq_moltip_temp_2
					clrf                freq_moltip_temp_3
					clrf                freq_moltip_temp_4
					clrf                freq_moltip_temp_5
;-------------------MOLTIPLICAZIONE SETTIMA CIFRA--------------------------------------------------------------------------------------------------
					movlw				M0
					mulwf				freq_ins_6
				    movff				PRODL,freq_moltip_temp_0
				    
				    movlw				M1
					mulwf				freq_ins_6
				    movff				PRODL,freq_moltip_temp_1
				    
				    movlw				M2
					mulwf				freq_ins_6
				    movff				PRODL,freq_moltip_temp_2
				    
				    movlw				M3
					mulwf				freq_ins_6
				    movff				PRODL,freq_moltip_temp_3
				    
				    movlw				M4
					mulwf				freq_ins_6
				    movff				PRODL,freq_moltip_temp_4
;-------------------RIPORTI SETTIMA CIFRA----------------------------------------------------------------------------------------------------------
				    decf                freq_moltip_temp_1                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_0,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPg			movwf               freq_moltip_temp_0                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_1                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_0,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPg                                      ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_0,W
					addwf				freq_moltip_3
					
					decf                freq_moltip_temp_2                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_1,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPg_2		movwf               freq_moltip_temp_1                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_2                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_1,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPg_2                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_1,W
					addwf				freq_moltip_4
		
					decf                freq_moltip_temp_3                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_2,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPg_3 		movwf               freq_moltip_temp_2                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_3                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_2,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPg_3                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_2,W
					addwf				freq_moltip_5
					
		            decf                freq_moltip_temp_4                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_3,W                            ;sposto in w per equilibrare il ciclo
				    
CICLO_RIPg_4 		movwf               freq_moltip_temp_3                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_4                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_3,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPg_4                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_3,W
					addwf				freq_moltip_6
		            
					decf                freq_moltip_temp_5                              ;decremento per equilibrare il ciclo
				    movf                freq_moltip_temp_4,W                            ;sposto in w per equilibrare il ciclo
		    
CICLO_RIPg_5		movwf               freq_moltip_temp_4                              ;salvo il valore della sottrazione  (quando ritorno dopo i riporti)
		            incf                freq_moltip_temp_5                              ;incremento il secondo temporaneo   (quando ritorno dopo i riporti)
		            
		            movlw				h'A'
					subwf				freq_moltip_temp_4,W
					
					btfsc               STATUS,C                                        ;se C=0 (valore negativo) esci dal ciclo
		            goto                CICLO_RIPg_5                                    ;altrimenti torna all'inizio del cico, incrementa il secondo temporaneo e salva il risultato della sotrazione nel primo temporaneo
					
					movf				freq_moltip_temp_4,W
					addwf				freq_moltip_7									;sommo il valore ottenuto al terzo byte del risultato
		
					movf				freq_moltip_temp_5,W
					addwf				freq_moltip_8
						
					call				CTRL_RIP_RISULT									;controllo i ripirti sul risultato, se lo faccio dopo ogni operazione al massimo avr? un riporto da gestire (9+9 max)
;-------------------ARROTONDAMENTO ALL'UNITA'------------------------------------------------------------------------------------------------------
					movlw				h'5'											;se freq_moltip <6 (cio >=5)
					cpfslt				freq_moltip_d_2
					incf				freq_moltip_0									;incrementa il primo valore
		
					call				CTRL_RIP_RISULT
		
					return
;-------------COTROLLO RIPORTI SUL RISULTATO-------------------------------------------------------------------------------------------------------
CTRL_RIP_RISULT		movlw				h'9'											;se freq_moltip >9 (cio <=10)
					cpfsgt				freq_moltip_d_0									;va bene cosi, salta e controlla il prossimo valore
					goto				CTRL_d_2
		
					movlw				h'A'
					subwf				freq_moltip_d_0,F								;altrimenti freq_moltip = freq_moltip-10
					incf				freq_moltip_d_1									;e incremento il valore successivo

CTRL_d_2			movlw				h'9'
					cpfsgt				freq_moltip_d_1
					goto				CTRL_d_3
		
					movlw				h'A'
					subwf				freq_moltip_d_1,F
					incf				freq_moltip_d_2

CTRL_d_3			movlw				h'9'
					cpfsgt				freq_moltip_d_2
					goto				CTRL_1
		
					movlw				h'A'
					subwf				freq_moltip_d_2,F
					incf				freq_moltip_0
		
CTRL_1				movlw				h'9'
					cpfsgt				freq_moltip_0
					goto				CTRL_2
					
					movlw				h'A'	
					subwf				freq_moltip_0,F
					incf				freq_moltip_1
		
CTRL_2				movlw				h'9'
					cpfsgt				freq_moltip_1
					goto				CTRL_3
		
					movlw				h'A'
					subwf				freq_moltip_1,F
					incf				freq_moltip_2
		
CTRL_3				movlw				h'9'
					cpfsgt				freq_moltip_2
					goto				CTRL_4
		
					movlw				h'A'
					subwf				freq_moltip_2,F
					incf				freq_moltip_3
		
CTRL_4				movlw				h'9'
					cpfsgt				freq_moltip_3
					goto				CTRL_5
		
					movlw				h'A'
					subwf				freq_moltip_3,F
					incf				freq_moltip_4
		
CTRL_5				movlw				h'9'
					cpfsgt				freq_moltip_4
					goto				CTRL_6
		
					movlw				h'A'
					subwf				freq_moltip_4,F
					incf				freq_moltip_5
		
CTRL_6				movlw				h'9'
					cpfsgt				freq_moltip_5
					goto				CTRL_7
		
					movlw				h'A'
					subwf				freq_moltip_5,F
					incf				freq_moltip_6
		
CTRL_7				movlw				h'9'
					cpfsgt				freq_moltip_6
					goto				CTRL_8
		
					movlw				h'A'
					subwf				freq_moltip_6,F
					incf				freq_moltip_7
		
CTRL_8				movlw				h'9'
					cpfsgt				freq_moltip_7
					return
		
					movlw				h'A'
					subwf				freq_moltip_7,F
					incf				freq_moltip_8
		
					return
;
;**************************************************************************************************************************************************
;   FUNCTION:       BCD9_TO_HEX4
;   PreCondition:   aver eseguito MOLTIP
;   Input:          freq_moltip_0 freq_moltip_1 freq_moltip_2 freq_moltip_3 freq_moltip_4 freq_moltip_5 freq_moltip_6 freq_moltip_7
;                   freq_moltip_8 (freq_moltip_0: LSB)
;   Output:         freq_hex_0 freq_hex_1 freq_hex_2 freq_hex_3 (freq_hex_0: LSB)
;   Overview:       Convrte 9 byte BCD (risultato della moltiplicazione contenuti in freq_moltip_x) in 4 byte esadecimali (freq_hex_x)
;**************************************************************************************************************************************************
;
BCD9_TO_HEX4		movff       		freq_moltip_0,freq_hex_0     					;la prima cifra sar sempre da mettere nel primo Byte cosi com'
					
					movf	   			freq_moltip_1,W
					call                BCD9_TO_HEX4_2
					
					movf    			freq_moltip_2,W
					call                BCD9_TO_HEX4_3
					
					movf        		freq_moltip_3,W
					call                BCD9_TO_HEX4_4
					
					movf       			freq_moltip_4,W
					call                BCD9_TO_HEX4_5
					
					movf        		freq_moltip_5,W
					call                BCD9_TO_HEX4_6
					
					movf        		freq_moltip_6,W
					call                BCD9_TO_HEX4_7
					
					movf        		freq_moltip_7,W
					call                BCD9_TO_HEX4_8
					
					movf       			freq_moltip_8,W
					call                BCD9_TO_HEX4_9

					return
;
;**************************************************************************************************************************************************
;BCD9_TO_HEX4_2: converte la seconda cifra della moltip in valore esadecimale diviso nei 4 byte (la seconda cifra influisce solo sul primo byte)
;**************************************************************************************************************************************************
;
BCD9_TO_HEX4_2     	movwf       		temp
		            xorlw				h'0'
		            btfsc   		    STATUS,Z        								;se  0
		            return                      										;torna senza fare nulla
		            
		            movf        		temp,W
		            xorlw				h'1'
					btfsc    	    	STATUS,Z        								;se  1 vai a gestire la prima conversione
		            goto       			BCD9_TO_HEX4_2_1
		            
		            movf        		temp,W
		            xorlw				h'2'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_2_2
		            
		            movf        		temp,W
		            xorlw				h'3'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_2_3
		            
		            movf        		temp,W
		            xorlw				h'4'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_2_4
		            
		            movf        		temp,W
		            xorlw				h'5'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_2_5
		            
		          	movf        		temp,W
		            xorlw				h'6'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_2_6
		            
		            movf        		temp,W
		            xorlw				h'7'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_2_7
		            
		            movf        		temp,W
		            xorlw				h'8'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_2_8
		            
					movf        		temp,W
		            xorlw				h'9'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_2_9
                                             
BCD9_TO_HEX4_2_1      								  									;10->A
		            movlw       		h'A'              								;sommo il valore in esadecimale (con il suo peso) al primo byte  
		            addwf       		freq_hex_0
		            return                       										;e ritorno, non sono necessari controlli di overflow perch il massimo che posso avere a questo punto  9+A

BCD9_TO_HEX4_2_2                          												;20->14
		            movlw       		h'14'             								;sommo il valore in esadecimale (con il suo peso) al primo byte 
		            addwf       		freq_hex_0
		            return                      										;e ritorno, non sono necessari controlli di overflow perchE il massimo che posso avere a questo punto  9+14
            
BCD9_TO_HEX4_2_3                         												;30->1E
		            movlw       		h'1E'          		    						;sommo il valore in esadecimale (con il suo peso) al primo byte 
		            addwf       		freq_hex_0
		            return                     		    								;e ritorno, non sono necessari controlli di overflow

BCD9_TO_HEX4_2_4                         												;40->28
		            movlw       		h'28'         		    						;sommo il valore in esadecimale (con il suo peso) al primo byte 
		            addwf       		freq_hex_0
		            return                    		    								;e ritorno, non sono necessari controlli di overflow
          
BCD9_TO_HEX4_2_5                         												;50->32
		            movlw      			h'32'         		    						;sommo il valore in esadecimale (con il suo peso) al primo byte 
		            addwf       		freq_hex_0
		            return                    		    								;e ritorno, non sono necessari controlli di overflow
            
BCD9_TO_HEX4_2_6                        			    								;60->3C
		            movlw       		h'3C'           								;sommo il valore in esadecimale (con il suo peso) al primo byte 
		            addwf       		freq_hex_0
		            return                      										;e ritorno, non sono necessari controlli di overflow
                                               
BCD9_TO_HEX4_2_7                          												;70->46
		            movlw      		 	h'46'             								;sommo il valore in esadecimale (con il suo peso) al primo byte 
		            addwf   		    freq_hex_0
		            return                       										;e ritorno, non sono necessari controlli di overflow
            
BCD9_TO_HEX4_2_8                          												;80->50
		            movlw      		 	h'50'       		    						;sommo il valore in esadecimale (con il suo peso) al primo byte 
		            addwf   		    freq_hex_0
		            return                       										;e ritorno, non sono necessari controlli di overflow
            
BCD9_TO_HEX4_2_9                          												;90->5A
		            movlw       		h'5A'             								;sommo il valore in esadecimale (con il suo peso) al primo byte 
		            addwf       		freq_hex_0
		            return                        										;e ritorno, non sono necessari controlli di overflow   
;
;**************************************************************************************************************************************************
;BCD9_TO_HEX4_3: converte la terza cifra della moltip in valore esadecimale diviso nei 4 byte (la seconda cifra influisce solo sul primo byte)
;**************************************************************************************************************************************************
; 
BCD9_TO_HEX4_3  	movwf       		temp
		            xorlw				h'0'
		            btfsc       		STATUS,Z        								;se  0
		            return                      										;torna senza fare nulla
		            
		            movf        		temp,W
		            xorlw				h'1'
					btfsc       		STATUS,Z        								;se  1 vai a gestire la prima conversione
		            goto        		BCD9_TO_HEX4_3_1
		            
		            movf        		temp,W
		            xorlw				h'2'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_3_2
		            
		            movf        		temp,W
		            xorlw				h'3'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_3_3
		            
		            movf        		temp,W
		            xorlw				h'4'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_3_4
		            
		            movf        		temp,W
		            xorlw				h'5'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_3_5
		            
		          	movf        		temp,W
		            xorlw				h'6'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_3_6
		            
		            movf        		temp,W
		            xorlw				h'7'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_3_7
		            
		            movf        		temp,W
		            xorlw				h'8'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_3_8
		            
					movf        		temp,W
		            xorlw				h'9'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_3_9
            
BCD9_TO_HEX4_3_1                         			 									;100->64
		            movlw       		h'64'            								;sommo il valore in esadecimale (con il suo peso) al primo byte  
		            addwf       		freq_hex_0
		            return                       										;e ritorno, non sono necessari controlli di overflow perchE il massimo che posso avere a questo punto  9+5A+64=C7

BCD9_TO_HEX4_3_2                        		 										;200->C8
		            movlw       		h'C8'            								;sommo il valore in esadecimale (con il suo peso) al primo byte  
		            addwf       		freq_hex_0
		 	        btfsc       		STATUS,C         								;se  andato in overflow
		            incf        		freq_hex_1       								;incrementa il secondo byte, posso avere massimo 9+5A+C8=12B  incremento incremento il secondo byte
		            return                                                                               

BCD9_TO_HEX4_3_3                        		 										;300->12C
		            movlw       		h'2C'            								;sommo il byte basso del valore in esadecimale (con il suo peso) al primo byte
		            addwf       		freq_hex_0       								;il massimo che posso avere  9+5A+2C=8F, non servono riporti
		            movlw       		h'01'            								;sommo il byte alto del valore in esadecimale al secondo byte
		            addwf       		freq_hex_1       								;non servono controlli
		            return

BCD9_TO_HEX4_3_4                  										       		  	;400->190
		            movlw       		h'90'            								;sommo il byte basso del valore in esadecimale (con il suo peso) al primo byte
		            addwf       		freq_hex_0       								;il massimo che posso avere  9+5A+90=F3, non servono riporti
		            movlw       		h'01'            								;sommo il byte alto del valore in esadecimale al secondo byte
		            addwf       		freq_hex_1       								;non servono controlli
		            return
        
BCD9_TO_HEX4_3_5                          		 										;500->1F4
		            movlw      		 	h'F4'           								;sommo il byte basso del valore in esadecimale (con il suo peso) al primo byte
		            addwf      		 	freq_hex_0      								;il massimo che posso avere E 9+5A+F4=157
		            btfsc      		 	STATUS,C        								;se c' riporto
		            incf       		 	freq_hex_1      								;incrementa il secondo byte
		            movlw      		 	h'01'           								;sommo il byte alto del valore in esadecimale al secondo byte
		            addwf      		 	freq_hex_1      								;non servono controlli
		            return 

BCD9_TO_HEX4_3_6                         		 										;600->258
		            movlw       		h'58'            								;sommo il byte basso del valore in esadecimale (con il suo peso) al primo byte
		            addwf       		freq_hex_0       								;il massimo che posso avere E 9+5A+58=BB, non servono riporti
		            movlw       		h'02'            								;sommo il byte alto del valore in esadecimale al secondo byte
		            addwf       		freq_hex_1       								;non servono controlli
		            return  

BCD9_TO_HEX4_3_7                        		 										;700->2BC
		            movlw       		h'BC'            								;sommo il byte basso del valore in esadecimale (con il suo peso) al primo byte
		            addwf       		freq_hex_0       								;il massimo che posso avere  9+5A+BC=11F
		            btfsc       		STATUS,C         								;se c' riporto
		            incf        		freq_hex_1       								;incrementa il secondo byte
		            movlw       		h'02'            								;sommo il byte alto del valore in esadecimale al secondo byte
		            addwf       		freq_hex_1       								;non servono controlli
		            return 

BCD9_TO_HEX4_3_8                         		 										;800->320
		            movlw       		h'20'            								;sommo il byte basso del valore in esadecimale (con il suo peso) al primo byte
		            addwf       		freq_hex_0       								;il massimo che posso avere  9+5A+20=83
		            movlw       		h'03'            								;sommo il byte alto del valore in esadecimale al secondo byte
		            addwf       		freq_hex_1       								;non servono controlli
		            return 		

BCD9_TO_HEX4_3_9                         		 										;900->384
		            movlw       		h'84'            								;sommo il byte basso del valore in esadecimale (con il suo peso) al primo byte
		            addwf       		freq_hex_0       								;il massimo che posso avere E 9+5A+84=E7
		            movlw       		h'03'            								;sommo il byte alto del valore in esadecimale al secondo byte
		            addwf       		freq_hex_1       								;non servono controlli
		            return    
;
;**************************************************************************************************************************************************
;BCD9_TO_HEX4_4
;**************************************************************************************************************************************************
; 
BCD9_TO_HEX4_4  	movwf       		temp
		          	xorlw				h'0'
		            btfsc       		STATUS,Z        								;se  0
		            return                      										;torna senza fare nulla
		            
		            movf        		temp,W
		            xorlw				h'1'
					btfsc       		STATUS,Z        								;se  1 vai a gestire la prima conversione
		            goto        		BCD9_TO_HEX4_4_1
		            
		            movf        		temp,W
		            xorlw				h'2'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_4_2
		            
		            movf        		temp,W
		            xorlw				h'3'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_4_3
		            
		            movf        		temp,W
		            xorlw				h'4'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_4_4
		            
		            movf        		temp,W
		            xorlw				h'5'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_4_5
		            
		          	movf        		temp,W
		            xorlw				h'6'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_4_6
		            
		            movf        		temp,W
		            xorlw				h'7'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_4_7
		            
		            movf        		temp,W
		            xorlw				h'8'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_4_8
		            
					movf        		temp,W
		            xorlw				h'9'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_4_9
            
BCD9_TO_HEX4_4_1                          			 									;1000->3E8
		            movlw       		h'E8'            								;sommo il valore in esadecimale (con il suo peso) al primo byte  
		            addwf       		freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
		            movlw				h'03'
					addwf				freq_hex_1
					return                       

BCD9_TO_HEX4_4_2                 														;2000->7D0
		            movlw       		h'D0'            
		            addwf       		freq_hex_0
		            btfsc       		STATUS,C         
		            incf        		freq_hex_1
					movlw				h'07'
					addwf				freq_hex_1
		            return                                                                               

BCD9_TO_HEX4_4_3    																	;3000->BB8
		            movlw       		h'B8'            
		            addwf       		freq_hex_0
		            btfsc       		STATUS,C         
		            incf        		freq_hex_1
		            movlw				h'B'
					addwf				freq_hex_1
					return

BCD9_TO_HEX4_4_4                      		  	 										;4000->FA0
		            movlw       		h'A0'            
		            addwf       		freq_hex_0
      				btfsc       		STATUS,C         
		            incf        		freq_hex_1
		            movlw       		h'0F'            
		            addwf       		freq_hex_1
		            return
        
BCD9_TO_HEX4_4_5                          			 									;5000->1388
		            movlw       		h'88'            
		            addwf       		freq_hex_0
		            btfsc       		STATUS,C         
		            incf        		freq_hex_1
		            movlw       		h'13'            
		            addwf       		freq_hex_1
		            return 

BCD9_TO_HEX4_4_6                         		 										;6000->1770
		            movlw       		h'70'            
		            addwf       		freq_hex_0
		            btfsc       		STATUS,C         
		            incf        		freq_hex_1
		            movlw       		h'17'            
		            addwf       		freq_hex_1
		            return 

BCD9_TO_HEX4_4_7                    	   		 										;7000->1B58
		            movlw       		h'58'            
		            addwf       		freq_hex_0
		            btfsc       		STATUS,C         
		            incf        		freq_hex_1
		            movlw       		h'1B'            
		            addwf       		freq_hex_1
		            return 

BCD9_TO_HEX4_4_8                         		 										;8000->1F40
		            movlw       		h'40'            
		            addwf       		freq_hex_0
		            btfsc       		STATUS,C         
		            incf        		freq_hex_1
		            movlw       		h'1F'            
		            addwf       		freq_hex_1
		            return 

BCD9_TO_HEX4_4_9                         		 										;9000->2328
		            movlw       		h'28'            
		            addwf       		freq_hex_0
		            btfsc       		STATUS,C         
		            incf        		freq_hex_1
		            movlw       		h'23'            
		            addwf       		freq_hex_1
		            return     
;
;**************************************************************************************************************************************************
;BCD9_TO_HEX4_5:
;**************************************************************************************************************************************************
; 
BCD9_TO_HEX4_5  	movwf       		temp
		            xorlw				h'0'
		            btfsc       		STATUS,Z        								;se  0
		            return                      										;torna senza fare nulla
		            
		            movf        		temp,W
		            xorlw				h'1'
					btfsc       		STATUS,Z        								;se  1 vai a gestire la prima conversione
		            goto        		BCD9_TO_HEX4_5_1
		            
		            movf        		temp,W
		            xorlw				h'2'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_5_2
		            
		            movf        		temp,W
		            xorlw				h'3'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_5_3
		            
		            movf        		temp,W
		            xorlw				h'4'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_5_4
		            
		            movf        		temp,W
		            xorlw				h'5'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_5_5
		            
		          	movf        		temp,W
		            xorlw				h'6'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_5_6
		            
		            movf        		temp,W
		            xorlw				h'7'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_5_7
		            
		            movf        		temp,W
		            xorlw				h'8'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_5_8
		            
					movf        		temp,W
		            xorlw				h'9'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_5_9
            
BCD9_TO_HEX4_5_1                       			 									    ;10000->2710
		            movlw				h'10'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					movlw				h'27'
					addwf				freq_hex_1
					return
	
BCD9_TO_HEX4_5_2                 														;20000->4E20
		           	movlw				h'20'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					movlw				h'4E'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					return                                                                              

BCD9_TO_HEX4_5_3    																	;30000->7530
		            movlw				h'30'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					movlw				h'75'
					addwf				freq_hex_1
					return      		         

BCD9_TO_HEX4_5_4                      		  	 										;40000->9C40
		            movlw				h'40'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					movlw				h'9C'
					addwf				freq_hex_1
					return               
        
BCD9_TO_HEX4_5_5                       			 									    ;50000->C350
		            movlw				h'50'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					movlw				h'C3'
					addwf				freq_hex_1
					return               
BCD9_TO_HEX4_5_6                         		 										;60000->EA60
		            movlw				h'60'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					movlw				h'EA'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					return               

BCD9_TO_HEX4_5_7                    	   		 										;70000->11170
		            movlw				h'70'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					movlw				h'11'
					addwf				freq_hex_1
					movlw				h'01'
					addwf				freq_hex_2
					return               

BCD9_TO_HEX4_5_8                         		 										;80000->13880
		            movlw				h'80'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					movlw				h'38'
					addwf				freq_hex_1
					movlw				h'1'
					addwf				freq_hex_2
					return 

BCD9_TO_HEX4_5_9                         		 										;90000->15F90
		            movlw				h'90'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					movlw				h'5F'
					addwf				freq_hex_1
					movlw				h'01'
					addwf				freq_hex_2
					return 

;
;**************************************************************************************************************************************************
;BCD9_TO_HEX4_6:
;**************************************************************************************************************************************************
; 
BCD9_TO_HEX4_6  	movwf       		temp
		            xorlw				h'0'
		            btfsc       		STATUS,Z        								;se  0
		            return                      										;torna senza fare nulla
		            
		            movf        		temp,W
		            xorlw				h'1'
					btfsc       		STATUS,Z        								;se  1 vai a gestire la prima conversione
		            goto        		BCD9_TO_HEX4_6_1
		            
		            movf        		temp,W
		            xorlw				h'2'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_6_2
		            
		            movf        		temp,W
		            xorlw				h'3'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_6_3
		            
		            movf        		temp,W
		            xorlw				h'4'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_6_4
		            
		            movf        		temp,W
		            xorlw				h'5'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_6_5
		            
		          	movf        		temp,W
		            xorlw				h'6'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_6_6
		            
		            movf        		temp,W
		            xorlw				h'7'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_6_7
		            
		            movf        		temp,W
		            xorlw				h'8'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_6_8
		            
					movf        		temp,W
		            xorlw				h'9'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_6_9
            
BCD9_TO_HEX4_6_1                       			 									    ;100000->186A0
		            movlw				h'A0'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'86'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'1'				
					addwf				freq_hex_2
					return
	
BCD9_TO_HEX4_6_2                 														;200000->30D40
		           	movlw				h'40'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'0D'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'03'				
					addwf				freq_hex_2
					return                                                           

BCD9_TO_HEX4_6_3    																	;300000->493E0
		            movlw				h'E0'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'93'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'4'				
					addwf				freq_hex_2
					return                                                           
               
BCD9_TO_HEX4_6_4                      		  	 										;400000->61A80
		            movlw				h'80'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'1A'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'6'				
					addwf				freq_hex_2
					return                                                           

        
BCD9_TO_HEX4_6_5                       			 									    ;500000->7A120
		            movlw				h'20'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'A1'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'07'				
					addwf				freq_hex_2
					return                                                           
             
BCD9_TO_HEX4_6_6                         		 										;60000->927C0
		            movlw				h'C0'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'27'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'9'				
					addwf				freq_hex_2
					return                                                           

BCD9_TO_HEX4_6_7                    	   		 										;700000->AAE60
		            movlw				h'60'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'AE'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'A'
					addwf				freq_hex_2
					return               

BCD9_TO_HEX4_6_8                         		 										;800000->C3500
		            movlw				h'35'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'C'
					addwf				freq_hex_2
					return 

BCD9_TO_HEX4_6_9                         		 										;900000->DBBA0
		            movlw				h'A0'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'BB'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'D'
					addwf				freq_hex_2
					return 
;
;**************************************************************************************************************************************************
;BCD9_TO_HEX4_7
;**************************************************************************************************************************************************
; 
BCD9_TO_HEX4_7  	movwf       		temp
		            xorlw				h'0'
		            btfsc       		STATUS,Z        								;se  0
		            return                      										;torna senza fare nulla
		            
		            movf        		temp,W
		            xorlw				h'1'
					btfsc       		STATUS,Z        								;se  1 vai a gestire la prima conversione
		            goto        		BCD9_TO_HEX4_7_1
		            
		            movf        		temp,W
		            xorlw				h'2'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_7_2
		            
		            movf        		temp,W
		            xorlw				h'3'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_7_3
		            
		            movf        		temp,W
		            xorlw				h'4'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_7_4
		            
		            movf        		temp,W
		            xorlw				h'5'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_7_5
		            
		          	movf        		temp,W
		            xorlw				h'6'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_7_6
		            
		            movf        		temp,W
		            xorlw				h'7'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_7_7
		            
		            movf        		temp,W
		            xorlw				h'8'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_7_8
		            
					movf        		temp,W
		            xorlw				h'9'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_7_9
            
BCD9_TO_HEX4_7_1                       			 									    ;1000000->F4240
		            movlw				h'40'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'42'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'F'				
					addwf				freq_hex_2
					return
	
BCD9_TO_HEX4_7_2                 														;2000000->1E8480
		           	movlw				h'80'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'84'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'1E'				
					addwf				freq_hex_2
					return                                                           

BCD9_TO_HEX4_7_3    																	;3000000->2DC6C0
		            movlw				h'C0'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'C6'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'2D'				
					addwf				freq_hex_2
					return                                                           
               
BCD9_TO_HEX4_7_4                      		  	 										;4000000->3D0900
		            movlw				h'09'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'3D'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					return                                                           

        
BCD9_TO_HEX4_7_5                       			 									    ;5000000->4C4B40
		            movlw				h'40'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'4B'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'4C'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					return                                                           
             
BCD9_TO_HEX4_7_6                         		 										;60000->5B8D80
		            movlw				h'80'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'8D'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'5B'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					return                                                        

BCD9_TO_HEX4_7_7                    	   		 										;7000000->6ACFC0
		            movlw				h'C0'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'CF'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'6A'
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					return               

BCD9_TO_HEX4_7_8                         		 										;800000->7A1200
		            movlw				h'12'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'7A'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					return

BCD9_TO_HEX4_7_9                         		 										;900000->895440
		            movlw				h'40'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'54'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'89'
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					return 
;
;**************************************************************************************************************************************************
;BCD9_TO_HEX4_8:
;**************************************************************************************************************************************************
; 
BCD9_TO_HEX4_8  	movwf       		temp
		            xorlw				h'0'
		            btfsc       		STATUS,Z        								;se  0
		            return                      										;torna senza fare nulla
		            
		            movf       			temp,W
		            xorlw				h'1'
					btfsc       		STATUS,Z        								;se  1 vai a gestire la prima conversione
		            goto        		BCD9_TO_HEX4_8_1
		            
		            movf        		temp,W
		            xorlw				h'2'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_8_2
		            		
		            movf        		temp,W
		            xorlw				h'3'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_8_3
		            
		            movf        		temp,W
		            xorlw				h'4'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_8_4
		            
		            movf        		temp,W
		            xorlw				h'5'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_8_5
		            
		          	movf        		temp,W
		            xorlw				h'6'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_8_6
		            
		            movf        		temp,W
		            xorlw				h'7'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_8_7
		            
		            movf        		temp,W
		            xorlw				h'8'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_8_8
		            
					movf        		temp,W
		            xorlw				h'9'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_8_9
            
BCD9_TO_HEX4_8_1                       			 									;10000000->989680
		            movlw				h'80'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					movlw				h'96'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'98'
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					return
	
BCD9_TO_HEX4_8_2                 														;20000000->1312D00
		           	movlw				h'2D'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'31'
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'1'
					addwf				freq_hex_3
					return                                               

BCD9_TO_HEX4_8_3    																	;30000000->1C9C380
		            movlw				h'80'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'C3'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'C9'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'1'
					addwf				freq_hex_3
					return                                                           
               
BCD9_TO_HEX4_8_4                      		  	 										;40000000->2625A00
		            movlw				h'5A'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'62'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'2'
					addwf				freq_hex_3
					return                                                             

        
BCD9_TO_HEX4_8_5                       			 									;50000000->2FAF080
		            movlw				h'80'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'F0'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'FA'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'2'
					addwf				freq_hex_3
					return                                                 
             
BCD9_TO_HEX4_8_6                         		 										;60000000->3938700
		            movlw				h'87'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'93'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'3'
					addwf				freq_hex_3
					return                                                           

BCD9_TO_HEX4_8_7                    	   		 										;70000000->42C1D80
		            movlw				h'80'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'1D'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'2C'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'4'
					addwf				freq_hex_3
					return             

BCD9_TO_HEX4_8_8                         		 										;80000000->1C4B400
		            movlw				h'B4'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'C4'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'4'
					addwf				freq_hex_3
					return

BCD9_TO_HEX4_8_9                         		 										;9000000->55D4A80
		            movlw				h'80'
					addwf				freq_hex_0
					btfsc       		STATUS,C         
		            incf        		freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'4A'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'5D'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'5'
					addwf				freq_hex_3
					return     

;
;**************************************************************************************************************************************************
;BCD9_TO_HEX4_9:
;**************************************************************************************************************************************************
; 
BCD9_TO_HEX4_9  	movwf       		temp
		            xorlw				h'0'
		            btfsc       		STATUS,Z        								;se  0
		            return                      										;torna senza fare nulla
		            
		            movf        		temp,W
		            xorlw				h'1'
					btfsc       		STATUS,Z        								;se  1 vai a gestire la prima conversione
		            goto        		BCD9_TO_HEX4_9_1
		            
		            movf        		temp,W
		            xorlw				h'2'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_9_2
		            
		            movf        		temp,W
		            xorlw				h'3'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_9_3
		            
		            movf        		temp,W
		            xorlw				h'4'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_9_4
		            
		            movf        		temp,W
		            xorlw				h'5'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_9_5
		            
		          	movf        		temp,W
		            xorlw				h'6'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_9_6
		            
		            movf        		temp,W
		            xorlw				h'7'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_9_7
		            
		            movf        		temp,W
		            xorlw				h'8'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_9_8
		            
					movf        		temp,W
		            xorlw				h'9'
					btfsc       		STATUS,Z        
		            goto        		BCD9_TO_HEX4_9_9
            
BCD9_TO_HEX4_9_1                      			 										;100000000->5F5E100
					movlw				h'E1'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'F5'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'5'
					addwf				freq_hex_3
					return

BCD9_TO_HEX4_9_2              										        			;200000000->BEBC200
					movlw				h'C2'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'EB'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'B'
					addwf				freq_hex_3
					return

BCD9_TO_HEX4_9_3                       			 									    ;300000000->11E1A300
					movlw				h'A3'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'E1'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'11'
					addwf				freq_hex_3
					return

BCD9_TO_HEX4_9_4                      			 										;400000000->17D78400
					movlw				h'84'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'D7'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'17'
					addwf				freq_hex_3
					return

BCD9_TO_HEX4_9_5                       			 									    ;500000000->1DCD65
					movlw				h'65'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'CD'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'1D'
					addwf				freq_hex_3
					return

BCD9_TO_HEX4_9_6                       			 									    ;600000000->23C34600
					movlw				h'46'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'C3'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'23'
					addwf				freq_hex_3
					return

BCD9_TO_HEX4_9_7                       			 									    ;700000000->29B92700
					movlw				h'27'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'B9'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'29'
					addwf				freq_hex_3
					return

BCD9_TO_HEX4_9_8                       			 									    ;800000000->2FAF0800
					movlw				h'08'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'AF'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'2F'
					addwf				freq_hex_3
					return

BCD9_TO_HEX4_9_9                      			 										;900000000->35A4E900
					movlw				h'E9'
					addwf				freq_hex_1
					btfsc       		STATUS,C         
		            incf        		freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'A4'				
					addwf				freq_hex_2
					btfsc       		STATUS,C         
		            incf        		freq_hex_3
					movlw				h'35'
					addwf				freq_hex_3
					return
;
;**************************************************************************************************************************************************
;   FUNCTION:       HEX1_TO_BCD3
;   Input:          hex_data
;   Output:         bcd_unit, bcd_dec, bcd_cent
;   Overview:       Converte il byte esadecimale contenuto in hex_data in 3 byte BCD bcd_unit, bcd_dec, bcd_cent
;**************************************************************************************************************************************************
;
HEX1_TO_BCD3        clrf                bcd_unit
                    clrf                bcd_dec
                    clrf                bcd_cent

                    movff               hex_data,bcd_unit

HEX1_TO_BCD3_LOOP   movlw               h'A'
                    subwf               bcd_unit,W
                    btfss               STATUS,C
                    return

                    movwf               bcd_unit
                    incf                bcd_dec

                    movf                bcd_dec,W
                    xorlw               h'0A'
                    btfss               STATUS,Z
                    goto                HEX1_TO_BCD3_LOOP

                    clrf                bcd_dec
                    incf                bcd_cent
                    goto                HEX1_TO_BCD3_LOOP
;
;**************************************************************************************************************************************************
;   FUNCTION:       RESET_VAR_CALC
;   Overview:       Resetta tutti i valori della moltiplicazione
;**************************************************************************************************************************************************
;
RESET_VAR_CALC  	clrf				freq_moltip_0
					clrf				freq_moltip_1
					clrf				freq_moltip_2
					clrf				freq_moltip_3
					clrf				freq_moltip_4
					clrf				freq_moltip_5
					clrf				freq_moltip_6
					clrf				freq_moltip_7
					clrf				freq_moltip_8

					clrf				freq_moltip_d_0
					clrf				freq_moltip_d_1
					clrf				freq_moltip_d_2

					clrf                freq_moltip_temp_0
					clrf                freq_moltip_temp_1
					clrf                freq_moltip_temp_2
					clrf                freq_moltip_temp_3
					clrf                freq_moltip_temp_4
					clrf                freq_moltip_temp_5

					clrf				freq_hex_0
					clrf				freq_hex_1
					clrf				freq_hex_2
					clrf				freq_hex_3

                    clrf                bcd_unit
                    clrf                bcd_dec
                    clrf                bcd_cent
                    clrf                hex_data
					return