;**************************************************************************************************************************************************
;   LIBRARY:      LETTURA E SCRITTURA EEPROM E TABELLE
;   FILE:         Memory_and_table.asm
;   LAST REV:     1.0
;	AUTHOR:       Ivo Colleoni
;  	DEVICE:       PIC18F2550
;	CREATED:	  16/02/2013
;	UPDATED:	  16/02/2013
;
;	DESCRIP:      Gestisce la lettura e la scirttura nella memoria EEPROM e nelle tabelle in memoria programma
;
;   FUNCTION:     READ_EE
;                 WRITE_EE
;                 READ_TAB
;                 RESET_VAR_MEMORY
;
;--------------------------------------------------------------------------------------------------------------------------------------------------
;   Change History:
;  	Rev.    Date                Changes
;   1.0     16/02/2013          - prima build
;
;**************************************************************************************************************************************************
;dichiarazione variabili, memoria allocata da 0x24 a 0x27
                    CBLOCK              0x24
                    ee_adr
                    ee_data
                    tab_adr
                    tab_data
                    ENDC
;
;**************************************************************************************************************************************************
;   FUNCTION:       READ_EE
;   Input:          ee_adr
;   Output:         ee_data
;   Overview:       sottoprogramma per la lettura dei dati in memoria EEPROM (legge il dato contenuto all'indirizzo ee_adr e lo salva in ee_data)
;**************************************************************************************************************************************************
;
READ_EE             movff               ee_adr,EEADR                                    ;imposto l'indirizzo da leggere
                    bcf                 EECON1,EEPGD
                    bcf                 EECON1,CFGS
                    bsf                 EECON1,RD                                       ;leggo il dato
                    movff               EEDATA,ee_data                                  ;metto il dato nella variabile
                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       WRITE_EE
;   Input:          ee_adr, ee_data
;   Overview:       sottoprogramma per la scrittura dei dati in memoria EEPROM (scrive ee_data all'indirizzo contenuto in ee_adr)
;**************************************************************************************************************************************************
;
WRITE_EE
                    bcf                 PIR2,EEIF                                       ;resetto il flag di scrittura completata
                    movff               ee_adr,EEADR                                    ;imposto l'indirizzo
                    movff               ee_data,EEDATA                                  ;sposto il dato da scrivere all'interno di EEDATA
                    bcf                 EECON1,EEPGD
                    bcf                 EECON1,CFGS
                    bsf                 EECON1,WREN                                     ;abilito la scrittura
                    movlw               h'55'
                    movwf               EECON2
                    movlw               0AAh
                    movwf               EECON2
                    bsf                 EECON1,WR                                       ;Inizio la scrittura
EEIF_CYC            btfss               PIR2,EEIF                                       ;scrittura completata?
                    goto                EEIF_CYC                               			;no
                    bcf                 EECON1,WREN                                     ;disabilito la scrittura
                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       WRITE_EE
;   Precondition:   inserire la tabella alla fine del file principale
;   Input:          tab_adr,
;   Output:         tab_data
;   Overview:       sottoprogramma per la lettura dei dati in tabella legge il dato contenuto all'indirizzo 7F + tab_adrx2 e lo salva in tab_data)
;**************************************************************************************************************************************************
;
READ_TAB            clrf                TBLPTRU
                    movlw               h'7F'
                    movwf               TBLPTRH
                    movf                tab_adr,W
                    rlncf               WREG,W
                    movwf               TBLPTRL
                    tblrd*
                    movff               TABLAT,tab_data
                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       RESET_VAR_MEMORY
;   Overview:       Resetta tutte le variabili utilizzate per la lettura e scrittura
;**************************************************************************************************************************************************
;
RESET_VAR_MEMORY    clrf                ee_adr
                    clrf                ee_data
                    clrf                tab_adr
                    clrf                tab_data
                    return